duration_from_response

Function duration_from_response 

pub fn duration_from_response(
    status: StatusCode,
    headers: &HeaderMap,
    backoff: &mut ExponentialBackoff,
) -> Option<Duration>
Return Some if the response specifies a Retry-After header or the provided backoff has another backoff to try. If None is returned, the request should not be retried.