pub struct FibonacciBuilder { /* private fields */ }Expand description
FibonacciBuilder is used to build a [FibonacciBackoff] which offers a delay with Fibonacci-based retries.
Default
- jitter: false
- min_delay: 1s
- max_delay: 60s
- max_times: 3
Examples
use anyhow::Result;
use backon::FibonacciBuilder;
use backon::Retryable;
async fn fetch() -> Result<String> {
Ok(reqwest::get("https://www.rust-lang.org")
.await?
.text()
.await?)
}
#[tokio::main(flavor = "current_thread")]
async fn main() -> Result<()> {
let content = fetch.retry(FibonacciBuilder::default()).await?;
println!("fetch succeeded: {}", content);
Ok(())
}Implementations
Sourceimpl FibonacciBuilder
impl FibonacciBuilder
Sourcepub const fn new() -> FibonacciBuilder
pub const fn new() -> FibonacciBuilder
Create a new FibonacciBuilder with default values.
Sourcepub const fn with_jitter(self) -> FibonacciBuilder
pub const fn with_jitter(self) -> FibonacciBuilder
Set the jitter for the backoff.
When jitter is enabled, FibonacciBackoff will add a random jitter between (0, current_delay) to the delay.
Sourcepub fn with_jitter_seed(self, seed: u64) -> FibonacciBuilder
pub fn with_jitter_seed(self, seed: u64) -> FibonacciBuilder
Set the seed value for the jitter random number generator. If no seed is given, a random seed is used in std and default seed is used in no_std.
Sourcepub const fn with_min_delay(self, min_delay: Duration) -> FibonacciBuilder
pub const fn with_min_delay(self, min_delay: Duration) -> FibonacciBuilder
Set the minimum delay for the backoff.
Sourcepub const fn with_max_delay(self, max_delay: Duration) -> FibonacciBuilder
pub const fn with_max_delay(self, max_delay: Duration) -> FibonacciBuilder
Set the maximum delay for the current backoff.
The delay will not increase if the current delay exceeds the maximum delay.
Sourcepub const fn without_max_delay(self) -> FibonacciBuilder
pub const fn without_max_delay(self) -> FibonacciBuilder
Set no maximum delay for the backoff.
The delay will keep increasing.
The delay will saturate at Duration::MAX which is an unrealistic delay.
Sourcepub const fn with_max_times(self, max_times: usize) -> FibonacciBuilder
pub const fn with_max_times(self, max_times: usize) -> FibonacciBuilder
Set the maximum number of attempts for the current backoff.
The backoff will stop if the maximum number of attempts is reached.
Sourcepub const fn without_max_times(self) -> FibonacciBuilder
pub const fn without_max_times(self) -> FibonacciBuilder
Set no maximum number of attempts for the current backoff.
The backoff will not stop by itself.
The backoff could stop reaching usize::MAX attempts but this is unrealistic.