Module settings

Module settings 

Models and utilities for managing QCS settings.

Structs

Applications
Settings for secondary applications used by QCS SDKs.
AuthServer
OAuth 2.0 authorization server.
Profile
A particular profile of Settings, which defines all the configurable options for connecting to a particular QCS instance using a particular set of credentials.
Pyquil
Settings for secondary applications used by pyquil.
Settings
The structure of QCS settings, typically serialized as a TOML file at DEFAULT_SETTINGS_PATH.

Constants

DEFAULT_SETTINGS_PATH
The default path that Settings will be loaded from;
SETTINGS_PATH_VAR
Setting the QCS_SETTINGS_FILE_PATH environment variable will change which file is used for loading Settings.