#[non_exhaustive]pub enum LoginError {
Pkce(PkceLoginError),
Device(DeviceLoginError),
Discovery(DiscoveryError),
Request(Error),
}Expand description
Errors that can occur when attempting to perform an interactive login.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Pkce(PkceLoginError)
Error that occurred while performing the Authorization Code (PKCE) flow.
Device(DeviceLoginError)
Error that occurred while performing the Device Authorization flow.
Discovery(DiscoveryError)
Error that occurred while fetching the discovery document from the OAuth2 issuer.
Request(Error)
Error that occurred while making http requests.