ExternallyManaged

Struct ExternallyManaged 

pub struct ExternallyManaged { /* private fields */ }
A struct that manages access tokens by utilizing a user-provided refresh function.

The ExternallyManaged struct allows users to define custom logic for fetching or refreshing access tokens.

Implementations

Source

impl ExternallyManaged

pub fn new( refresh_function: impl Fn(AuthServer) -> Pin<Box<dyn Future<Output = Result<String, Box<dyn Error + Send + Sync>>> + Send>> + Send + Sync + 'static, ) -> Self

Creates a new ExternallyManaged instance from a RefreshFunction.

Consider using ExternallyManaged::from_async, and ExternallyManaged::from_sync, if they better fit your use case.

Arguments
  • refresh_function - A function or closure that asynchronously refreshes a token.
Example
use qcs_api_client_common::configuration::{settings::AuthServer, tokens::ExternallyManaged, TokenError};
use std::future::Future;
use std::pin::Pin;
use std::boxed::Box;
use std::error::Error;

async fn example_refresh_function(_auth_server: AuthServer) -> Result<String, Box<dyn Error
+ Send + Sync>> {
    Ok("new_token_value".to_string())
}
let token_manager = ExternallyManaged::new(|auth_server| Box::pin(example_refresh_function(auth_server)));
pub fn from_async<F, Fut>(refresh_function: F) -> Self
where F: Fn(AuthServer) -> Fut + Send + Sync + 'static, Fut: Future<Output = Result<String, Box<dyn Error + Send + Sync>>> + Send + 'static,

Constructs a new ExternallyManaged instance using an async function or closure.

This method simplifies the creation of the ExternallyManaged instance by handling the boxing and pinning of the future internally.

Arguments
Example
use qcs_api_client_common::configuration::{settings::AuthServer, tokens::ExternallyManaged, TokenError};
use tokio::runtime::Runtime;
use std::error::Error;

async fn example_refresh_function(_auth_server: AuthServer) -> Result<String, Box<dyn Error
+ Send + Sync>> {
    Ok("new_token_value".to_string())
}

let token_manager = ExternallyManaged::from_async(example_refresh_function);

let rt = Runtime::new().unwrap();
rt.block_on(async {
    match token_manager.request_access_token(&AuthServer::default()).await {
        Ok(token) => println!("Token: {token:?}"),
        Err(e) => println!("Failed to refresh token: {:?}", e),
    }
});
pub fn from_sync( refresh_function: impl Fn(AuthServer) -> Result<String, Box<dyn Error + Send + Sync>> + Send + Sync + 'static, ) -> Self

Constructs a new ExternallyManaged instance using a synchronous function.

The synchronous function is wrapped in an async block to fit the expected signature.

Arguments
Example
use qcs_api_client_common::configuration::{settings::AuthServer, tokens::ExternallyManaged, TokenError};
use tokio::runtime::Runtime;
use std::error::Error;

fn example_sync_refresh_function(_auth_server: AuthServer) -> Result<String, Box<dyn Error
+ Send + Sync>> {
    Ok("sync_token_value".to_string())
}

let token_manager = ExternallyManaged::from_sync(example_sync_refresh_function);

let rt = Runtime::new().unwrap();
rt.block_on(async {
    match token_manager.request_access_token(&AuthServer::default()).await {
        Ok(token) => println!("Token: {token:?}"),
        Err(e) => println!("Failed to refresh token: {:?}", e),
    }
});
pub async fn request_access_token( &self, auth_server: &AuthServer, ) -> Result<SecretAccessToken, Box<dyn Error + Send + Sync>>

Request an updated access token using the provided refresh function.

Errors

Errors are propagated from the refresh function.

Source

impl ExternallyManaged

pub fn __repr__(&self) -> String

Implements __repr__ for Python in terms of the Rust Debug implementation.