pub struct ExponentialBuilder { /* private fields */ }Expand description
ExponentialBuilder is used to construct an [ExponentialBackoff] that offers delays with exponential retries.
Default
- jitter: false
- factor: 2
- min_delay: 1s
- max_delay: 60s
- max_times: 3
Examples
use anyhow::Result;
use backon::ExponentialBuilder;
use backon::Retryable;
async fn fetch() -> Result<String> {
Ok(reqwest::get("https://www.rust-lang.org")
.await?
.text()
.await?)
}
#[tokio::main(flavor = "current_thread")]
async fn main() -> Result<()> {
let content = fetch.retry(ExponentialBuilder::default()).await?;
println!("fetch succeeded: {}", content);
Ok(())
}Implementations
Sourceimpl ExponentialBuilder
impl ExponentialBuilder
Sourcepub const fn new() -> ExponentialBuilder
pub const fn new() -> ExponentialBuilder
Create a new ExponentialBuilder with default values.
Sourcepub const fn with_jitter(self) -> ExponentialBuilder
pub const fn with_jitter(self) -> ExponentialBuilder
Enable jitter for the backoff.
When jitter is enabled, [ExponentialBackoff] will add a random jitter within (0, current_delay)
to the current delay.
Sourcepub fn with_jitter_seed(self, seed: u64) -> ExponentialBuilder
pub fn with_jitter_seed(self, seed: u64) -> ExponentialBuilder
Set the seed value for the jitter random number generator. If no seed is given, a random seed is used in std and default seed is used in no_std.
Sourcepub const fn with_factor(self, factor: f32) -> ExponentialBuilder
pub const fn with_factor(self, factor: f32) -> ExponentialBuilder
Set the factor for the backoff.
Note: Having a factor less than 1.0 does not make any sense as it would create a
smaller negative backoff.
Sourcepub const fn with_min_delay(self, min_delay: Duration) -> ExponentialBuilder
pub const fn with_min_delay(self, min_delay: Duration) -> ExponentialBuilder
Set the minimum delay for the backoff.
Sourcepub const fn with_max_delay(self, max_delay: Duration) -> ExponentialBuilder
pub const fn with_max_delay(self, max_delay: Duration) -> ExponentialBuilder
Set the maximum delay for the backoff.
The delay will not increase if the current delay exceeds the maximum delay.
Sourcepub const fn without_max_delay(self) -> ExponentialBuilder
pub const fn without_max_delay(self) -> ExponentialBuilder
Set no maximum delay for the backoff.
The delay will keep increasing.
The delay will saturate at Duration::MAX which is an unrealistic delay.
Sourcepub const fn with_max_times(self, max_times: usize) -> ExponentialBuilder
pub const fn with_max_times(self, max_times: usize) -> ExponentialBuilder
Set the maximum number of attempts for the current backoff.
The backoff will stop if the maximum number of attempts is reached.
Sourcepub const fn without_max_times(self) -> ExponentialBuilder
pub const fn without_max_times(self) -> ExponentialBuilder
Set no maximum number of attempts for the current backoff.
The backoff will not stop by itself.
The backoff could stop reaching usize::MAX attempts but this is unrealistic.
Sourcepub const fn with_total_delay(
self,
total_delay: Option<Duration>,
) -> ExponentialBuilder
pub const fn with_total_delay( self, total_delay: Option<Duration>, ) -> ExponentialBuilder
Set the total delay for the backoff.
The backoff will stop yielding sleep durations once the cumulative sleep time
plus the next sleep duration would exceed total_delay.